# This files contains your custom actions which can be used to run
# custom Python code.
#
# See this guide on how to implement these action:
# https://rasa.com/docs/rasa/custom-actions



from numbers import Number
from typing import Any, Text, Dict, List
from datetime import date, datetime, timedelta
from xml import dom
import requests
from sqlalchemy import DateTime, Float, false
import json

from rasa_sdk import Action, Tracker
from rasa_sdk.events import SlotSet, AllSlotsReset, FollowupAction, ReminderScheduled
from rasa_sdk.executor import CollectingDispatcher

#mattermost_webhook_url = "http://192.168.0.145:8065/hooks/9j1csgyn37dz3jy4ixrbotsb4o"
#rasa_webhook_url = "http://192.168.0.145:5007/webhooks/mattermostConnector/webhook"

# Als Action registriert damit es am Anfang initialisiert wird
class SetUpConfigFile(Action):
    def name(self) -> Text:
        with open('editable_config.json') as data_file:
            data_loaded = json.load(data_file)
        global mattermost_webhook_url
        mattermost_webhook_url = data_loaded["mattermost_webhook_url"]
        global rasa_webhook_url
        rasa_webhook_url = data_loaded["rasa_webhook_url"]
        print("Mattermost_Webhook: " + str(mattermost_webhook_url))
        print("Rasa Webhook:  " + str(rasa_webhook_url))
        return "read_config"

        
class SetUsernameAction(Action):
    def name(self) -> Text:
        return "action_set_username"

    def run(self, dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: Dict[Text, Any]) -> List[Dict[Text, Any]]:

        sender_id = tracker.current_state()['sender_id']
        print("Set slot username for " + sender_id)
        return [SlotSet('username', sender_id)]

class AskForReadyness(Action):
    def name(self) -> Text:
        return "action_ask_for_readyness"

    def run (self, 
            dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: Dict[Text, Any]) -> List[Dict[Text, Any]]:
        username = tracker.current_state()['sender_id']
        message = "Einen wunderschönen guten Tag, " + username + "! Hätten Sie etwas Zeit für eine kurze Reflexionssitzung?"
        attachments=self.getAttachementReminder()
        OutgoingWebhook.send_via_webhook(message=message, receiver=username, attachment=attachments)
        return    

    def getAttachementReminder(self) -> List[Dict[Text, Any]]:
        return [
                    {
                    "actions": [
                        {
                            "name": "Ja, liebend gerne!",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "start",
                                }
                            }
                        },
                        {
                            "name": "In 5 Minuten?",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/i_want_5_min"
                                }
                            }
                        }
                    ]
                    }
                ] 
class RemindFiveMin(Action):
    def name(self) -> Text:
        return "action_remind_five_min"

    def run(self, 
        dispatcher: CollectingDispatcher,
        tracker: Tracker,
        domain: Dict[Text, Any]) -> List[Dict[Text, Any]]:
        # Ermöglicht den Nutzer, einen neuen Reminder in 5min zu setzen
        username = tracker.current_state()['sender_id']
        reminder_date = datetime.now()
        reminder_distance = timedelta(minutes=5)
        reminder_date = reminder_date + reminder_distance
        
        # Reguläre Erinnerung
        events = []
        events.append(
                ReminderScheduled(
                    intent_name = 'start_reflection_session', 
                    trigger_date_time = reminder_date, 
                    name = "reflection_session", 
                    kill_on_user_message= True))
        text = "Sie werden in 5 Minuten erneut erinnert: " + reminder_date.strftime("%d-%b-%Y (%H:%M:%S)") 
        OutgoingWebhook.send_via_webhook(message=text, receiver=username, attachment=[] )

        return events

class AskWeekday(Action):
    def name(self) -> Text:
        return "action_ask_weekday"
    def run(self, 
        dispatcher: CollectingDispatcher,
        tracker: Tracker,
        domain: Dict[Text, Any]) -> List[Dict[Text, Any]]:
        print(str("Setze Woche"))
        username = tracker.current_state()['sender_id']
        text = "Welcher Wochentag wäre denn Ihnen genehm?"
        attachments = [
                    {
                    "actions": [
                        {
                            "name": "Montag",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_weekday{\"weekday\" : \"Montag\"}",
                                }
                            }
                        },
                        {
                            "name": "Dienstag",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_weekday{\"weekday\" : \"Dienstag\"}",
                                }
                            }
                        },
                        {
                            "name": "Mittwoch",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_weekday{\"weekday\" : \"Mittwoch\"}",
                                }
                            }
                        },
                        {
                            "name": "Donnerstag",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_weekday{\"weekday\" : \"Donnerstag\"}",
                                }
                            }
                        },
                        {
                            "name": "Freitag",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_weekday{\"weekday\" : \"Freitag\"}",
                                }
                            }
                        },
                        {
                            "name": "Jetzt",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_weekday{\"weekday\" : \"jetzt\"}",
                                }
                            }
                        }
                    ]
                    }
                ]
        OutgoingWebhook.send_via_webhook(message=text, receiver=username, attachment=attachments)

        return

class AskTime(Action):
    def name(self) -> Text:
        return "action_ask_time"
    def run(self, 
        dispatcher: CollectingDispatcher,
        tracker: Tracker,
        domain: Dict[Text, Any]) -> List[Dict[Text, Any]]:
        print(str("Setze Zeit"))
        username = tracker.current_state()['sender_id']
        weekday = tracker.get_slot("weekday")
        time = tracker.get_slot("time")
        events = []
        if time == None:
            text = "Welche Uhrzeit würde am " + weekday + " passen?"
            OutgoingWebhook.send_via_webhook(message=text, receiver=username, attachment=self.getTimeAttachement() )
        else:
            setReminder = SetNewReminderForThisWeek()
            events = setReminder.run(dispatcher=dispatcher, tracker=tracker, domain=domain)

        return events
    def getTimeAttachement(self) -> List[Dict[Text, Any]]:
        return [
                    {
                    "actions": [
                        {
                            "name": "8:00",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"8:00\"}",
                                }
                            }
                        },                        
                        {
                            "name": "8:30",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"8:30\"}",
                                }
                            }
                        },
                        {
                            "name": "9:00",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"9:00\"}",
                                }
                            }
                        },
                        {
                            "name": "9:30",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"9:30\"}",
                                }
                            }
                        },
                        {
                            "name": "10:00",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"10:00\"}",
                                }
                            }
                        },
                        {
                            "name": "10:30",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"10:30\"}",
                                }
                            }
                        },
                        {
                            "name": "11:00",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"11:00\"}",
                                }
                            }
                        },
                        {
                            "name": "11:30",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"11:30\"}",
                                }
                            }
                        },
                        {
                            "name": "12:00",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"12:00\"}",
                                }
                            }
                        },
                        {
                            "name": "12:30",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"12:30\"}",
                                }
                            }
                        },
                        {
                            "name": "13:00",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"13:00\"}",
                                }
                            }
                        },
                        {
                            "name": "13:30",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"13:30\"}",
                                }
                            }
                        },
                        {
                            "name": "14:00",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"14:00\"}",
                                }
                            }
                        },
                        {
                            "name": "14:30",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"14:30\"}",
                                }
                            }
                        },
                        {
                            "name": "15:00",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"15:00\"}",
                                }
                            }
                        },
                        {
                            "name": "15:30",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"15:30\"}",
                                }
                            }
                        },
                        {
                            "name": "16:00",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"16:00\"}",
                                }
                            }
                        },
                        {
                            "name": "16:30",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"16:30\"}",
                                }
                            }
                        }, 
                        {
                            "name": "17:00",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "/specify_time{\"time\" : \"17:00\"}",
                                }
                            }
                        },                                                                                                 
                    ]
                    }
                ]

class SetNewReminderForThisWeek(Action):
    def name(self) -> Text:
        return "action_set_messagetime"

    def run(self, 
        dispatcher: CollectingDispatcher,
        tracker: Tracker,
        domain: Dict[Text, Any]) -> List[Dict[Text, Any]]:
        print(str("Setze Reminder"))
        events = []
        weekday = tracker.get_slot("weekday")
        time = tracker.get_slot("time")
        # Wenn Slots nicht befüllt sind, bitte befüllen
        if weekday==None:
            print(str("Frag mal Woche"))
            AskWeekday().run(dispatcher, tracker, domain)
            return
        elif weekday=="jetzt":
            events.append(SlotSet("weekday", None))
            events.append(SlotSet("time", None))
            time = datetime.now() + timedelta(seconds=5)
            print(str("Setze Reminder jetzt:") + time.strftime("%d-%b-%Y (%H:%M:%S)") + " Uhr")
            events.append(
            ReminderScheduled(
                intent_name = 'start_reflection_session', 
                trigger_date_time = time, 
                name = "reflection_session of now", 
                kill_on_user_message= True))
            return events
        if time==None:
            print(str("Frag mal Zeit"))
            AskTime().run(dispatcher, tracker, domain)
            return
        # Slots zurücksetzen
        events.append(SlotSet("weekday", None))
        events.append(SlotSet("time", None))
        time_array = str(time).split(":")
        hour1 = self.get_number(text = str(time_array[0]))
        minute1 = self.get_number(str(time_array[1]))

        weekdate1 = self.calculateNextDate(str(weekday), tracker.get_slot("number_of_session"))
        reminder_date1 = datetime(2022, weekdate1.month, weekdate1.day ,  hour1, minute1, 0, 851609)
        
        # Erinnerung an den oben bestimmten Daten setzen
        events.append(
            ReminderScheduled(
                intent_name = 'start_reflection_session', 
                trigger_date_time = reminder_date1, 
                name = "reflection_session of " + str(reminder_date1), 
                kill_on_user_message= True))
        
        text = "Erinnerung gesetzt am "+ reminder_date1.strftime("%d-%b-%Y (%H:%M:%S)") + " Uhr \n"
        OutgoingWebhook.send_via_webhook(message=text, receiver=tracker.current_state()['sender_id'], attachment=self.getAttachment() )


        return events

    def get_number(self, text: Text) -> Number:
        if text.isdigit:
            return int(text)
        else: 
            return -1
    def calculateNextDate(self, weekday: Text, number_of_session: Float) -> DateTime:
        now = datetime.now()
        weekdays = ["Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "samstag", "sonntag"]
        week_index = weekdays.index(weekday)
        now_index = now.weekday()
        if now_index < week_index and SessionInfo.doWeHaveToDoThisSessionInThisWeek(number_of_session):
            return now + timedelta(days = (week_index - now_index))
        else: 
            return now + timedelta(days = ( 7 + week_index - now_index))

    def getAttachment(self) -> List[Dict[Text, Any]]:
        return [{
            "actions": [{
                "name": "Ich hätte doch lieber gerne einen anderen Tag.",
                "integration": {
                    "url": rasa_webhook_url,
                    "context": {
                        "text": "/choose_another_day",
                    }
                }
            }]
        }] 
        
class OutgoingWebhook(Action):
    def name(self) -> Text:
        return "outgoing_webhook"

    @staticmethod
    def send_via_webhook(message: Text, receiver: Text, attachment: List[Dict[Text, Any]]):
        payload = {
            "channel": "@" + str(receiver), 
            "text": message,
            "icon_url": "https://i.ibb.co/XXrVwYV/robot-icons-30475.png",
            "username" : "Nachhaltiges Lernen",
            "attachments": attachment
            }
        x = requests.post(mattermost_webhook_url, json = payload)
        print("Sende an outgoing webhook an: " + receiver + " mit Antwort vom Server:" + x.text)
        return
    
    @staticmethod
    def getAttachementFinish() -> List[Dict[Text, Any]]:
        return [
                    {
                    "actions": [
                        {
                            "name": "Ich bin fertig!",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "weiter",
                                }
                            }
                        }
                    ]
                    }
                ]
    

        
class SubmitReflectionSessionAction(Action):

    def name(self) -> Text:
        return "action_submit_reflection_session"

    def run(self, dispatcher: CollectingDispatcher,
            tracker: Tracker,
            domain: Dict[Text, Any]) -> List[Dict[Text, Any]]:
        username = tracker.current_state()['sender_id']
        events = []
        events.append(SlotSet("judgement", None))
        events.append(SlotSet("emotions", None))
        events.append(SlotSet("learning", None))
        events.append(SlotSet("planning", None))

        number_of_session = tracker.get_slot("number_of_session") + 1.0
        events.append(SlotSet('number_of_session', number_of_session ))

        print("Number of Session: " + str(number_of_session))
        if number_of_session == 5.0:
            text= "Vielen Dank für die Teilnahme an dieser Studie! Wir bitten Sie zum Abschluss der Studie diese kurze Umfrage auszufüllen: https://userpage.fu-berlin.de/~besahre/survey/limesurvey/index.php/669661?lang=de"
        else:
            text= "Vielen Dank für das Teilen Ihrer Gedanken zur Situation. Ich freue mich auf das nächste Mal! Apropros, nächste Mal, mich würd interessieren wann ich Sie so anschreiben kann. Sollten diese Woche noch Sitzungen offen sein (von den 2 Sitzungen pro Woche) dann bezeichnen die Wochentage diese Woche, ansonsten nächste Woche."
        OutgoingWebhook.send_via_webhook(message=text, receiver=username, attachment=[] )
        AskWeekday().run(dispatcher, tracker, domain)

        return events

class AskForGroupAction(Action):
    def name(self) -> Text:
        return "action_ask_group_form_group"

    def run(
        self, dispatcher: CollectingDispatcher, tracker: Tracker, domain: Dict
    ) -> List[Dict[Text, Any]]:
        username = tracker.current_state()['sender_id']
        text = "Welcher Gruppe wurden Sie zugewiesen? Bitte klicken Sie auf den entsprechenden Button."
        OutgoingWebhook.send_via_webhook(message=text, receiver=username, attachment=self.getAttachementGroup() )
        return

    def getAttachementGroup(self) -> List[Dict[Text, Any]]:
        return [
                    {
                    "text": "Wählen Sie Ihre Gruppe aus.",
                    "actions": [
                        {
                            "name": "Gruppe 1",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "1",
                                }
                            }
                        },
                        {
                            "name": "Gruppe 2",
                            "integration": {
                                "url": rasa_webhook_url,
                                "context": {
                                    "text": "2",
                                }
                            }
                        }
                    ]
                    }
                ] 


class AskForJudgementAction(Action):
    def name(self) -> Text:
        return "action_ask_reflection_form_judgement"

    def run(
        self, dispatcher: CollectingDispatcher, tracker: Tracker, domain: Dict
    ) -> List[Dict[Text, Any]]:
        username = tracker.current_state()['sender_id']
        events = []
        # Wähle Name je nach Session
        name= SessionInfo.getNameForSession(tracker)
        # Je nach Namen passendes Pronomen wählen
        if name == "Eberhardt":
            pronoun = "den Assistenzarzt"
        else:
            pronoun = "die Assistenzärztin"
            
        OutgoingWebhook.send_via_webhook(
            message="Sie haben sich nun in " + pronoun + " " + name + " hineinversetzt und beantworten alles aus dieser Perspektive.", 
            receiver=username, 
            attachment=[] )
        if tracker.slots.get("group")=="1":
            text = "Guten Tag Dr. " + name + ",\nbitte nehmen Sie sich sieben Minuten Zeit, um über Ihre Situation nachzudenken. Schreiben Sie alle Ihre Gedanken in den Chat. \n"
            events.append(SlotSet("judgement", ""))
            events.append(SlotSet("emotions", ""))
            events.append(SlotSet("learning", ""))
            events.append(SlotSet("planning", ""))
        else:
            pretext = "Guten Tag Dr. " + name + ", \n ich helfe Ihnen gerne, über die Situation nachzudenken. Dazu würde ich Ihnen gerne vier Fragen stellen. \n"
            pretext += "Bitte nehmen Sie sich in Summe sieben Minuten Zeit, um über Ihre Situation mit Hilfe der Fragen nachzudenken."
            OutgoingWebhook.send_via_webhook(message=pretext, receiver=username, attachment=[])
            text = "Wie bewerten Sie Ihre Situation (siehe oben) und Ihr Handeln in der Situation?\n"

        text += "Wenn Sie alle Ihre Gedanken in den Chat geschrieben haben, drücken Sie auf \"Ich bin fertig!\". Alternativ können Sie auch \"/ich-denke ich bin fertig\" in den Chat schreiben."
        OutgoingWebhook.send_via_webhook(message=text, receiver=username, attachment=OutgoingWebhook.getAttachementFinish())
        return events

class AskForEmotionAction(Action):
    def name(self) -> Text:
        return "action_ask_reflection_form_emotions"

    def run(
        self, dispatcher: CollectingDispatcher, tracker: Tracker, domain: Dict
    ) -> List[Dict[Text, Any]]:
        username = tracker.current_state()['sender_id']

        name = SessionInfo.getNameForSession(tracker)
        text = name + ", welche Emotionen spüren Sie, wenn Sie sich in Ihre erlebte Situation zurückversetzen?\n"
        text += "Wenn Sie alle Ihre Gedanken in den Chat geschrieben haben, drücken Sie auf \"Ich bin fertig!\""
        OutgoingWebhook.send_via_webhook(message=text, receiver=username, attachment=OutgoingWebhook.getAttachementFinish())

        return []

class AskForLearningAction(Action):
    def name(self) -> Text:
        return "action_ask_reflection_form_learning"

    def run(
        self, dispatcher: CollectingDispatcher, tracker: Tracker, domain: Dict
    ) -> List[Dict[Text, Any]]:
        username = tracker.current_state()['sender_id']

        name = SessionInfo.getNameForSession(tracker)
        text = "Welche neue Erkenntnisse haben Sie, " + name + ", durch das Nachdenken über Ihre Situation erlangt?\n"
        text += "Wenn Sie alle Ihre Gedanken in den Chat geschrieben haben, bestätigen Sie mit mit Klick auf den Button."
        OutgoingWebhook.send_via_webhook(message=text, receiver=username, attachment=OutgoingWebhook.getAttachementFinish())
        return []

class AskForPlanningAction(Action):
    def name(self) -> Text:
        return "action_ask_reflection_form_planning"

    def run(
        self, dispatcher: CollectingDispatcher, tracker: Tracker, domain: Dict
    ) -> List[Dict[Text, Any]]:
        username = tracker.current_state()['sender_id']

        name= SessionInfo.getNameForSession(tracker)
        text = name + ", wie würden Sie jetzt anders handeln, wenn Sie in eine ähnliche Situation kommen sollten?\n"
        OutgoingWebhook.send_via_webhook(message=text, receiver=username, attachment=OutgoingWebhook.getAttachementFinish())
        return []

class ConfirmParticipationAction(Action):
    def name(self) -> Text:
        return "action_confirm_participation"
    def run(
        self, dispatcher: CollectingDispatcher, tracker: Tracker, domain: Dict
    ) -> List[Dict[Text, Any]]:
        username = tracker.current_state()['sender_id']
        group = tracker.get_slot("group")
        text = "Vielen Dank für die Teilnahme an der Studie! Sie befinden sich nun in Gruppe "+ group + ". Zu Beginn der Studie erhalten Sie über diesen Kanal eine Nachricht."

        OutgoingWebhook.send_via_webhook(message=text, receiver=username, attachment=OutgoingWebhook.getAttachementFinish())
        return []

class ReflectionThema(Action):
    def name(self) -> Text:
        return "action_reflection_thema"
    def run(
        self, dispatcher: CollectingDispatcher, tracker: Tracker, domain: Dict
    ) -> List[Dict[Text, Any]]:
        username = tracker.current_state()['sender_id']
        number_of_session = tracker.get_slot("number_of_session")
        text = self.getReflectionTheme(number_of_session)
        attachments = self.getAttachements(number_of_session)
        OutgoingWebhook.send_via_webhook(message=text, receiver=username, attachment=attachments)
        return

    def getAttachements(self, number_of_session: Float) -> List[Dict[Text, Any]]:
        if number_of_session==4.0:
            return [
                    {
                    "text": "Die von Eberhardt erhaltene Mail:",
                    "image_url": "https://i.ibb.co/bbZKgSN/2022-07-07-11-23-31-Widerherstellung-Passworts-erforderlich.png"
                    }
                ] 
        else:
            return []

    def getReflectionTheme(self, number_of_session: Float) -> Text:
        if number_of_session==0.0:
            return "Betrachten Sie folgende Situation. Versetzen Sie sich in den Assistenzarzt Eberhardt hinein und beantworten Sie anschließend die Fragen aus seiner Perspektive. \n Die elektronische Patientenakte (ePA) gibt es seit Anfang 2021. Diese soll die bessere Verfügbarkeit von Patientendaten gewährleisten. \n Der Assistenzarzt Eberhardt hat soeben das Ergebnis eines therapeutischen Gesprächs mit einer Patientin über die Ursachen ihrer Bulimia nervosa in die ePA hochgeladen.\n Dann liest er folgenden Zeitungsartikel, der ihm von einem Kollegen empfohlen wurde: \n\n\"In Finnland wurden Ende 2018 Patientendaten aus 20 Psychotherapiekliniken gestohlen. Nach erfolgloser Erpressung der Kliniken wurden tausende Patienten mit der Drohung erpresst, dass der Inhalt der therapeutischen Gespräche veröffentlicht werde, sollten die Betroffenen kein Lösegeld zahlen.\" \n\nDer Artikel lässt Eberhardt überlegen, welche Daten alle in die ePA hochgeladen werden sollten. "
        if number_of_session==1.0:
            return "Betrachten Sie folgende Situation. Versetzen Sie sich in die Assistenzärztin Vanessa hinein und beantworten Sie anschließend die Fragen aus ihrer Perspektive.\n Assistenzärztin Vanessa nimmt bei der Anamnese Daten ihres Patienten Alfred auf. Sie hat die Wahl, zu Forschungszwecken deutlich mehr Daten aufzunehmen als eigentlich zur Behandlung des Patienten notwendig.\n Zwar ist im Kleingedruckten des zusätzlichen Anamnesebogens angegeben, dass die Angaben für den Patienten optional sind und nicht explizit für die Behandlung benötigt werden, aber Vanessa hat ihre Zweifel ob der Patient angemessen darüber unterrichtet wurde.\n Sie beschließt, dass das nicht ihre Aufgabe ist und fährt mit der Behandlung fort."
        if number_of_session==2.0:
            return "Betrachten Sie folgende Situation. Versetzen Sie sich in die Assistenzärztin Vanessa hinein und beantworten Sie anschließend die Fragen aus ihrer Perspektive. \nAssistenzärztin Vanessa schreibt Assistenzarzt Eberhardt über WhatsApp wegen der Aufnahme von Forschungsdaten bei der Anamnese ihres Patienten. Eberhardt bestätigt ihre Bedenken und empfiehlt ein aufklärendes Gespräch bezüglich der Forschungsdaten, auch wenn das mehr Zeit beansprucht. \nAnschließend fragt Eberhardt Vanessa, ob er die psychologischen Gutachten seines Patienten in die elektronische Patientenakte hochladen sollte. Er beschreibt dabei genau, welche Inhalte im psychologischen Gutachten vorhanden sind, die er als kritisch sieht."
        if number_of_session==3.0:
            return "Betrachten Sie folgende Situation. Versetzen Sie sich in den Assistenzarzt Eberhardt hinein und beantworten Sie anschließend die Fragen aus seiner Perspektive.\nUm den Assistenzärzt:innen Zugriff auf das Computersystem von außen zu ermöglichen, wird durch die IT der Klinik ein VPN eingerichtet. \nAlle Assistenzärzt:innen können dadurch mit Name und Passwort von zu Hause aus auf das System um die enthaltenen vertraulichen Daten ihrer Patienten zugreifen. Assistenzärztin Vanessa wählt ein kompliziertes Passwort. Am nächsten Tag klagt sie darüber, dass sie ihr Passwort vergessen hat. \nDer Assistenzarzt Eberhardt kann darüber nur schmunzeln. Er wählte für seinen Account das Passwort \"12345\"."
        if number_of_session==4.0:
            return "Betrachten Sie folgende Situation. Versetzen Sie sich in den Assistenzarzt Eberhardt hinein und beantworten Sie anschließend die Fragen aus seiner Perspektive.\nDer Assistenzarzt Eberhardt bekommt eine E-Mail mit \"Wiederherstellung Ihres Passworts erforderlich\" im Betreff (siehe Bild unten).  \n\n Eberhardt klickt auf den in der Mail bereitgestellten Link und wird auf eine Website weitergeleitet, die genauso aussieht wie das Mattermost-System seiner Klinik. Allerdings enthält die URL oben im Browser weder das Wort Mattermost, noch ist das \"Schloss-Symbol\" sichtbar. Eberhardt gibt in die erforderlichen Felder sein Passwort ein und klickt auf \"Passwort wiederherstellen\". \nZwei Wochen später folgt eine Rundmail der IT, dass es einen Hacker-Angriff gegeben hat, bei dem Daten gestohlen wurden. Eberhardt denkt an die stattgefundene Situation zurück."
        if number_of_session==5.0:
            return "Betrachten Sie folgende Situation. Versetzen Sie sich in die Assistenzärztin Vanessa hinein und beantworten Sie anschließend die Fragen aus ihrer Perspektive.\nDie Assistenzärztin Vanessa meldet sich über das VPN in ihrer Klinik an, um sich ihren Terminkalender anzuschauen – der Chefarzt sprach mit ihr am Nachmittag über einen Termin, der angepasst wurde. \nBei der Anmeldung im VPN fällt ihr auf, dass sie sich auch ohne Passwort, nur mit Angabe ihres Benutzernamens, anmelden kann. Vanessa freut sich, weil sie sowieso ihr Passwort vergessen hatte. Vanessa weiß, dass sie morgen noch mehrfach über das VPN auf das System zugreifen muss, daher beschließt sie, die fehlende Passwortabfrage erst in zwei Tagen der IT zu melden."
        return "Vielen Dank für die Teilnahme an dieser Studie! Wir bitten Sie zum Abschluss der Studie diese kurze Umfrage auszufüllen: https://doctalk.limesurvey.net/545456?lang=de"



class SessionInfo(Action):
    def name(self) -> Text:
        return "action_session_info"
    @staticmethod
    def getNameForSession(tracker: Tracker) -> Text:
        number_of_session = tracker.slots.get("number_of_session")
        if number_of_session==0.0 or number_of_session==3.0 or number_of_session==4.0:
            return "Eberhardt"
        else:
            return "Vanessa"
    @staticmethod
    def doWeHaveToDoThisSessionInThisWeek(number_of_session: Float) -> bool:
        # startdates der 3 Wochen, in denen die Reflexionssitzungen stattfinden sollen
        startdates = [datetime(year=2022, month = 8, day = 1 ), datetime(year=2022, month = 8, day = 8), datetime(year=2022, month = 8, day = 15)]
        # Startdatum für die eingereicht number_of_session
        start_of_session = startdates[int(number_of_session/2)]
        # ist das startdatum verstrichen, sollte die eingereichte Session möglichst noch diese Woche stattfinden, sonst nächste
        return (datetime.now() > start_of_session)

